/* 
  NAME:   REGULATOR LED OSVETLENI SCHODISTE   
  MCU:    ATmega16
  
  PROJEKT MMIA, VUT 2013                              

*/
/* ********************************************** */

// knihovny
#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdio.h>
#include <util/delay.h>
#include "adc.h"    // knihovna pro A/D prevod

#define F_OSC 		8000000  // definice kmitoctu int. oscilatoru

// makra pro ovladani signalizacni LED
#define LED_OFF		(PORTC |= (1<<PC5))
#define LED_ON		(PORTC &= ~(1<<PC5))

#define POCET_LED	13    // pocet PWM vystupu 
#define T_SHUFFLE	200	  // casovy offset posuvu[us]

#define SOUMRAK_OK()			(ADCch3>ADCch5)  // kontrola okolni urovne osvetleni
#define AKT_VSTUP()				(bit_is_clear(PINB,2)) // aktivace vstupu /AKT
#define VYPOCTI_CAS_ODPOCTU()	(300+((ADCch2/2)*3)) // makro vypoctu casu 


volatile unsigned char pwm_request[POCET_LED]; 

// promenne pro uziti jednotlivych A/D prevodu
volatile uint16_t ADCch0;
volatile uint16_t ADCch1;
volatile uint16_t ADCch2;
volatile uint16_t ADCch3;
volatile uint16_t ADCch4;
volatile uint16_t ADCch5;

unsigned char last_akt = 0;  // pomocna promenna kontroly posledni aktivace

// preruseni od Timeru 1
ISR(TIMER0_COMP_vect)
{
	static unsigned char poloha = 0;
		
	poloha++;

	unsigned int i = 0;
	for (i=0; i<POCET_LED; i++)
	{
		if(pwm_request[i]>poloha)
		{
			if(i<8)	PORTD |= (1<<i);
			else	PORTC |= (1<<(i-8));		
		}
		else 
		{
			if(i<8)	PORTD &= ~(1<<i);
			else	PORTC &=  ~(1<<(i-8));		
		}
	}		

}	

// fce postupneho rozsveceni - prvni smer
void postupneRozni(unsigned int rychlost, unsigned int jas_postupny) {
		unsigned char i;

		rychlost = (unsigned int)((unsigned long)rychlost*256/jas_postupny);
		
		for(i=0;i<13;i++)	// rozsvecuj - smer 1
		{
			unsigned char j;
	
			if (AKT_VSTUP()) {
				last_akt = 123;
				return;
			}
					


			for(j=0;j<jas_postupny;j++)
			{
				pwm_request[i] = j;
				volatile unsigned int cek = rychlost;
				while (cek--);
			}					
		}
}


// fce postupneho rozsveceni - druhy smer
void postupneRozniNaopak(unsigned int rychlost, unsigned int jas_postupny) {
		unsigned char i;
		
		rychlost = (unsigned int)((unsigned long)rychlost*256/jas_postupny);

		for(i=13;i>0;i--)	// rozsvecuj - smer 2
		{
			unsigned char j;
	
			if (AKT_VSTUP()) {
				last_akt = 123;
				return;
			}
					
			
			for(j=0;j<jas_postupny;j++)
			{
				pwm_request[i-1] = j;
				volatile unsigned int cek = rychlost;
				while (cek--);
			}					
		}
}

// fce postupneho zhasnuti - vse soucasne
void postupneZhasni(unsigned int rychlost, unsigned int jas_postupny) {
		unsigned char i;
		unsigned char j;

		rychlost = (unsigned int)((unsigned long)rychlost*256/jas_postupny);

		for(j=jas_postupny;j>0;j--)
		{
			for(i=0;i<13;i++)	// zhasinej - vse soucasne
			{
				pwm_request[i] = j-1;
				
			}
			
			if (AKT_VSTUP()) {
				last_akt = 123;   // "nalhani" hodnoty last_akt
				return;
			}
					
			volatile unsigned int cek = rychlost*3;
			while (cek--);			
		}
}

//  vsecko soucasne rozni (pro AKT vstup)
void postupneRozniVseSoucasne(unsigned int rychlost, unsigned int jas) {
		unsigned char i;
		unsigned char j;

		rychlost = (unsigned int)((unsigned long)rychlost*256/jas);


		for(j=0;j<255;j++)
		{
			for(i=0;i<13;i++)	// rozihej - vse soucasne
			{
				if (pwm_request[i]<jas) pwm_request[i]++;
				if (pwm_request[i]>jas) pwm_request[i]--;
			}
					
			volatile unsigned int cek = rychlost*3;
			while (cek--);			
		}
}


// ------------ MAIN -----------------------

int main(void) {
	
  // nastaveni smeru portu
	DDRA = 0x00;
	DDRB = 0x00;
	DDRD = 0xff;
	DDRC = 0xff;
  // nastaveni nekterych pull-up
	PORTB = 0x07;
	
 

	// nastaveni Timer0
	TCCR0 = (1<<WGM01)|(1<<CS01);
	TCNT0=0x00;
	OCR0=0x18;	


	// povoleni preruseni od Timer0
	TIMSK = (1<<OCIE0);

	sei();  // povoleni glob. preruseni

  // nastaveni hodnot na portech (vystupy)
	PORTD = 0x00;
	PORTC = 0x00;

  // inicializace A/D prevodnich fci
	initAD();
	
  // pomocne promenne
	unsigned int odpocet = 0;
	unsigned char slowcnt = 0;

  // testovaci sekvence
	while ((bit_is_clear(PINB,0))||(bit_is_clear(PINB,1))) {
	  postupneRozni(50,255);
	  postupneZhasni(50,255);
	  postupneRozniNaopak(50,255);
	  postupneZhasni(50,255);
	}
	

while(1)
{

	while(/*(bit_is_set(PINB,0)) && (bit_is_set(PINB,1))*/ 1)	// (ne)aktivace od MW senzoru
	{	
		static unsigned char citac = 0;		

		switch(citac)					// vycitani AD prevodniku
		{
			case 0:{
				StartAD(CH0);
				ADCch0 = ReadAD(CH0);	// jas
				break;
				}
			case 1:{
				StartAD(CH1);
				ADCch1 = ReadAD(CH1);	// rychlost rozsvecovani
				break;
				}
			case 2:{
				StartAD(CH2);
				ADCch2 = ReadAD(CH2);	// doba svitu
				break;
				}
			case 3:{
				StartAD(CH3);
				ADCch3 = ReadAD(CH3);	// intenzita aktivace 
				break;
				}
			case 4:{
				StartAD(CH4);
				ADCch4 = ReadAD(CH4);	// citlivost
				break;
				}
			case 5:{
				StartAD(CH5);
				ADCch5 = ReadAD(CH5);	// fotorezistor
				break;
				}	
		}
		_delay_ms(10);
		citac++;
		if (citac>5) citac=0;


		slowcnt++;  // cnt pro LED

      unsigned int rychlost = 30+ADCch1;		// prepocet AD prevodu na rychlost rozsvecovani - nastaveni TR3
//		unsigned int odpocet = 1+(ADCch2/128);
		  unsigned int jas = ADCch0/4;			// nastaveni jasu pro AKT
		  unsigned int jas_postupny = ADCch4/4;	// nastaveni jasu pri postupn. rozsveceni
				
		if(SOUMRAK_OK() && (slowcnt&0x10)) LED_OFF;   // ovladani sign. LED
		else LED_ON;

		//VSTUP MW 1
		if(bit_is_clear(PINB,0)
			&& SOUMRAK_OK()
			&& !AKT_VSTUP()	
			) //rozsvecej smer nahoru
		{
			if (!odpocet) {
				postupneRozni(rychlost,jas_postupny);			//postupne rozsviti
			}
			odpocet = VYPOCTI_CAS_ODPOCTU();				//spusti resp. restartuje odpocet
		} 

		//VSTUP MW 2
		if(bit_is_clear(PINB,1)
			&& SOUMRAK_OK()
			&& !AKT_VSTUP()	
			) //rozsvecej smer dolu
		{
			if (!odpocet) postupneRozniNaopak(rychlost,jas_postupny);	//postupne rozsviti
			odpocet = VYPOCTI_CAS_ODPOCTU();				//spusti resp. restartuje odpocet
		} 



		//uplynuti casu
		if (odpocet) {
			odpocet--;
			if (!odpocet) {
				//prave ted uplynul cas odpoctu
				//pokud neni zapnuty akt. vstup
				//muzeme vsechno zhasnout
				//plynule zhasnuti
				postupneZhasni(rychlost,jas_postupny);
			}
		}

	
	// reakce na vstup /AKT
	if(AKT_VSTUP())
	{
		// sepnut
		if(last_akt!=1)
		{
			// ted to sepnulo
			last_akt = 1;
			odpocet = 0;
			postupneRozniVseSoucasne(rychlost, jas);
		}
	}
	else
	{
		// rozepnut
		if(last_akt!=0)
		{
			// ted rozepnulo
			last_akt = 0;
			odpocet = 0;
			postupneZhasni(rychlost,jas);
		}
	}	

	}

} // end loop 

return (0);
} // end main
